#!/usr/bin/env pythoon3
"""Read AVS dats file."""
import sys
from argparse import ArgumentParser
from pathlib import Path

try:
    import ITkDBlogin
    import ITkDButils
except ModuleNotFoundError:
    from itkdb_gtk import ITkDBlogin, ITkDButils

import dateutil.parser
import openpyxl as XL
from openpyxl.cell.cell import MergedCell
from openpyxl.utils.exceptions import InvalidFileException

sk_defaults = {
    "institution": "AVS",
    "runNumber": "1",
}


class AVSDataException(Exception):
    """AVSData exception class."""

    def __init__(self, message):
        """Call the base class constructor with the parameters it needs."""
        super().__init__(message)


def create_weight(session, SN, the_date=None, manager="", passed=True, problems=False, comments=[]):
    """Creates the dictionary for a WEIGHT test.

    Args:
    ----
        session: the DB session
        SN: Serial Number
        the_date: the date of the test
        manager: manager name
        passed: if test passed or not
        problems: if problems were found during test
        comments: list of comments to append to the test

    """
    the_date = ITkDButils.get_db_date(the_date)
    out = ITkDButils.get_test_skeleton(session, "CORE_PETAL", "WEIGHING", sk_defaults)
    out['component'] = SN
    out['date'] = the_date
    out['passed'] = passed
    out['problems'] = problems
    out['properties']['PRODUCTION_MANAGER'] = manager
    out['comments'] = comments
    return out


def create_manufacturing(session, SN, the_date=None, manager="", passed=True, problems=False, comments=[]):
    """Create the dictionary or the MANUFACTURING test.

    Args:
    ----
        session: the DB session
        SN: Serial Number
        the_date: the date of the test
        manager: manager name
        passed: if test passed or not
        problems: if problems were found during test
        comments: list of comments to append to the test

    """
    the_date = ITkDButils.get_db_date(the_date)
    out = ITkDButils.get_test_skeleton(session, "CORE_PETAL", "MANUFACTURING", sk_defaults)
    out['component'] = SN
    out['date'] = the_date
    out['passed'] = passed
    out['problems'] = problems
    out['properties']['PRODUCTION_MANAGER'] = manager
    out['comments'] = comments

    return out


def create_visual_inpection(session, SN, the_date=None, operator="", passed=True, problems=False, comments=[]):
    """Create Visual Inspection test skeleton."""
    the_date = ITkDButils.get_db_date(the_date)
    out = ITkDButils.get_test_skeleton(session, "CORE_PETAL", "VISUAL_INSPECTION", sk_defaults)
    out['component'] = SN
    out['date'] = the_date
    out['passed'] = passed
    out['problems'] = problems
    out['properties']['OPERATOR'] = operator
    out['comments'] = comments

    return out


def create_delamination_test(session, SN, the_date=None, operator="", passed=True, problems=False, comments=[]):
    """Create the delamination test JSON.

    Args:
    ----
        session: the DB session
        SN: Serial Number
        the_date: the date of the test
        operator: operator name
        passed: if test passed or not
        problems: if problems were found during test
        comments: list of comments to append to the test

    """
    the_date = ITkDButils.get_db_date(the_date)
    out = ITkDButils.get_test_skeleton(session, "CORE_PETAL", "DELAMINATION", sk_defaults, {"boolean": False})
    out['component'] = SN
    out['date'] = the_date
    out['passed'] = passed
    out['problems'] = problems
    out['properties']['OPERATOR'] = operator
    out['comments'] = comments

    return out


def create_grounding_test(session, SN, the_date=None, operator="", passed=True, problems=False, comments=[]):
    """Create grounding test.

    Args:
    ----
        session: the DB session
        SN: Serial Number
        the_date: the date of the test
        operator: operator name
        passed: if test passed or not
        problems: if problems were found during test
        comments: list of comments to append to the test

    """
    the_date = ITkDButils.get_db_date(the_date)
    out = ITkDButils.get_test_skeleton(session, "CORE_PETAL", "GROUNDING_CHECK", sk_defaults, {"boolean": False})
    out['component'] = SN
    out['date'] = the_date
    out['passed'] = passed
    out['problems'] = problems
    out['properties']['OPERATOR'] = operator
    out['comments'] = comments

    return out


def create_metrology_test(session, SN, the_date=None, operator="", passed=True, problems=False, comments=[]):
    """Metrology test.

    Args:
    ----
        session: the DB session
        SN: Serial Number
        the_date: the date of the test
        operator: operator name
        passed: if test passed or not
        problems: if problems were found during test
        comments: list of comments to append to the test

    """
    the_date = ITkDButils.get_db_date(the_date)
    out = ITkDButils.get_test_skeleton(session, "CORE_PETAL", "METROLOGY_AVS",
                                       sk_defaults, {"integer": -1, "float": -1.0})
    out['component'] = SN
    out['date'] = the_date
    out['passed'] = passed
    out['problems'] = problems
    out['properties']['OPERATOR'] = operator
    out['comments'] = comments

    return out


def split_comp_list(lst):
    """Split a  list of components separated by  various possible characters."""
    if lst is None:
        return []

    if isinstance(lst, float):
        return [lst]

    if isinstance(lst, int):
        return [lst]

    out = [lst]
    for sep in ['/', '\\', '\n']:
        if lst.find(sep) >= 0:
            out = [x.strip() for x in lst.split(sep)]
            break

    return out


def get_comments(txt):
    """Return test DB comment."""
    return split_comp_list(txt)


def get_float(cell, separator=None, default=0.0):
    """Return float from string."""
    txt = cell.value
    if txt is None:
        return default

    if separator is None:
        if isinstance(txt, float):
            return txt

        if isinstance(txt, int):
            return float(txt)

        return default

    else:
        values = []
        for val in split_comp_list(txt):
            if isinstance(val, float) or isinstance(val, int):
                values.append(val)
            else:
                try:
                    v = float(val.strip().replace(',', '.'))
                except ValueError:
                    print("get_float: Cannot convert {} in {}".format(val, cell.coordinate))
                    v = default

                values.append(v)

        return values


def get_boolean(cell):
    """Get a boolean from a cell."""
    value = cell.value
    if value is None:
        return False

    else:
        txt = value.strip().lower()
        return (txt == "pass")


def get_text(cell):
    """Get a string from a cell."""
    value = cell.value
    if value:
        value = value.strip()

    return value


def get_res_and_accep(sheet, indx):
    """Return result and acceptancee."""
    val = get_float(sheet["g{}".format(indx)])
    accept = get_boolean(sheet["h{}".format(indx)])
    return val, accept


def find_idx(lst, val):
    """Return index of occurence of val in lst."""
    idx = [i for i, x in enumerate(lst) if (x and x.find(val) >= 0)]
    return idx


def cell_value(sheet, coord):
    """Return the cell value."""
    cell = sheet[coord]
    if not isinstance(cell, MergedCell):
        return cell.value

    # "Oh no, the cell is merged!"
    for range in sheet.merged_cells.ranges:
        if coord in range:
            return range.start_cell.value

    return None


def check_for_problems(sheet, the_test, row_range):
    """Finds FAIL massages in the Acceptance column."""
    nfail = 0
    for row in range(row_range[0], row_range[1]):
        txt = get_text(sheet["h{}".format(row)])
        if txt is None:
            continue

        txt = txt.lower()
        if txt[0] == 'f':
            nfail += 1
            hdr = get_text(sheet["d{}".format(row)])
            reason = cell_value(sheet, "i{}".format(row))

            if len(reason) < 20:
                msg = "{}: {}".format(hdr, reason)
                the_test["defects"].append({"name": msg})
            else:
                the_test["defects"].append({"name": hdr,
                                            "description": reason})

    if nfail:
        the_test["passed"] = False


def get_coreID(bustapeID):
    """Build core SN from bus tape SN."""
    SN = "20USEBC" + bustapeID[-7:]
    return SN


def readFATfile(session, fnam, SN=None):
    """Read data from FAT excel file.

    Args:
    ----
        session: the DB session
        fnam: File path
        SN: COre serial number

    """
    # Open spreadsheet
    try:
        wb = XL.load_workbook(fnam)
    except InvalidFileException as ee:
        print("Could not open input file: ", fnam)
        print(ee)
        return None

    # Assume active sheet is the good one, otherwise will have to find in wb.sheetnames
    sheet = wb.active
    if sheet.max_row < 50 or sheet.max_column < 9:
        raise AVSDataException("Wrong FAT file")

    # Check the SN of the petal core
    if SN is None or len(SN) == 0:
        coreID = split_comp_list(sheet['C6'].value)
        if len(coreID) == 0:
            raise AVSDataException("Cannot figure out core SN in FAT file.")

        for cID in coreID:
            cmp = ITkDButils.get_DB_component(session, cID)
            if cmp["type"]["code"] == "BT_PETAL_FRONT":
                SN = get_coreID(cID)
                break

    batch = sheet['E6'].value
    operator = sheet['G6'].value

    txt = list(map(str.strip, sheet['i4'].value.split(':')))[1]
    test_date = dateutil.parser.parse(txt)

    # Get the test index
    test_name = [str(sheet[x][1].value) for x in range(1, sheet.max_row)]
    tests = [str(sheet[x][3].value) for x in range(1, sheet.max_row)]

    # This is to avoid adding 1 for cell names...
    tests.insert(0, None)
    test_name.insert(0, None)

    #
    # Visual inspection
    vi_text = get_text(sheet['i9'])
    vi_result = get_text(sheet['g9'])
    vi_pass = (sheet['h9'].value.strip().lower() == "pass")
    vi_defects = []
    if vi_pass:
        if vi_result and len(vi_result):
            if vi_text and len(vi_text):
                vi_text = vi_result + '\n' + vi_text
            else:
                vi_text = vi_result

    else:
        vi_defects.append({"name": "PETAL_VI_DEFECT", "description": vi_result})

    vi_test = create_visual_inpection(session, SN, test_date, operator, vi_pass,
                                      comments=get_comments(vi_text))
    for df in vi_defects:
        vi_test["defects"].append(df)

    #
    # Delamination test
    dl_text = get_text(sheet['i10'])
    dl_result = get_text(sheet['g10'])
    dl_pass = (sheet['h10'].value.strip().lower() == "pass")
    dl_defects = []
    if dl_pass:
        if dl_result and len(dl_result):
            if dl_text and len(dl_text):
                dl_text = dl_result + '\n' + dl_text
            else:
                dl_text = dl_result

    else:
        dl_defects.append({"name": "PETAL_DL_DEFECT",
                           "description": dl_result})

    delamination_test = create_delamination_test(session, SN, test_date, operator, dl_pass,
                                                 comments=get_comments(dl_text))
    for df in dl_defects:
        delamination_test["defects"].append(df)

    #
    # Conductivity
    # TODO: read proper rows
    grounding_test = create_grounding_test(session, SN, test_date, operator)
    cond_val, cond_pass = get_res_and_accep(sheet, tests.index("COND"))
    if "INS_LOOP" in tests:
        loop_val, loop_pass = get_res_and_accep(sheet, tests.index("INS_LOOP"))
    else:
        loop_val, loop_pass = get_res_and_accep(sheet, tests.index("INS"))

    if "INS_LOOP_GND" in tests:
        loop_gnd_val, loop_gnd_pass = get_res_and_accep(sheet, tests.index("INS_LOOP_GND"))
    else:
        loop_gnd_val, loop_gnd_pass = get_res_and_accep(sheet, tests.index("INS_FACE"))

    grounding_test["results"]["RESISTANCE_FB"] = cond_val
    grounding_test["results"]["RESISTANCE_PIPES"] = loop_val
    grounding_test["results"]["RESISTANCE_PIPE_GND"] = loop_gnd_val
    check_for_problems(sheet, grounding_test, [tests.index('COND'), tests.index("WEIGH")])

    #
    # Weight
    petal_weight, weight_pass = get_res_and_accep(sheet, tests.index("WEIGH"))

    #
    # Metrology AVS
    metrology_test = create_metrology_test(session, SN, test_date, operator)
    metrology_test["results"]["LOCATOR1_DIAMETER"] = get_float(sheet['g{}'.format(tests.index("PL01_DIAM"))])
    metrology_test["results"]["LOCATOR2_DIAMETER"] = get_float(sheet['g{}'.format(tests.index("PL02_DIAM"))])
    metrology_test["results"]["LOCATOR3_DIAMETER"] = get_float(sheet['g{}'.format(tests.index("PL03_DIAM"))])
    metrology_test["results"]["LOCATOR2_X"] = get_float(sheet['g{}'.format(find_idx(tests, "PL02_X")[0])])
    metrology_test["results"]["LOCATOR2_Y"] = get_float(sheet['g{}'.format(find_idx(tests, "PL02_Y")[0])])
    metrology_test["results"]["LOCATOR3_X"] = get_float(sheet['g{}'.format(find_idx(tests, "PL03_X")[0])])
    metrology_test["results"]["LOCATOR3_Y"] = get_float(sheet['g{}'.format(find_idx(tests, "PL03_Y")[0])])
    metrology_test["results"]["FIDUCIAL1_DIAMETER"] = get_float(sheet["g{}".format(find_idx(tests, "FD01_DIAM")[0])])
    metrology_test["results"]["FIDUCIAL1_X"] = get_float(sheet["g{}".format(find_idx(tests, "FD01_X")[0])])
    metrology_test["results"]["FIDUCIAL1_Y"] = get_float(sheet["g{}".format(find_idx(tests, "FD01_Y")[0])])
    metrology_test["results"]["FIDUCIAL2_DIAMETER"] = get_float(sheet["g{}".format(find_idx(tests, "FD02_DIAM")[0])])
    metrology_test["results"]["FIDUCIAL2_X"] = get_float(sheet["g{}".format(find_idx(tests, "FD02_X")[0])])
    metrology_test["results"]["FIDUCIAL2_Y"] = get_float(sheet["g{}".format(find_idx(tests, "FD02_Y")[0])])
    metrology_test["results"]["ANGLE_VCHANNEL"] = get_float(sheet["g{}".format(find_idx(tests, "VANGL")[0])])
    metrology_test["results"]["ENVELOPE"] = get_float(sheet["g{}".format(find_idx(tests, "ENVEL")[0])])
    metrology_test["results"]["COPLANARITY_FRONT"] = get_float(sheet["g{}".format(find_idx(tests, "F/PL_PLAN")[0])])
    metrology_test["results"]["LOCAL_FLATNESS_FRONT"] = get_float(sheet["g{}".format(find_idx(tests, "F/FS_PLAN")[0])], '/')
    metrology_test["results"]["PARALLELISM_FRONT"] = get_float(sheet["g{}".format(find_idx(tests, "F/PARAL")[0])])
    metrology_test["results"]["COPLANARITY_BACK"] = get_float(sheet["g{}".format(find_idx(tests, "B/PL_PLAN")[0])])
    metrology_test["results"]["LOCAL_FLATNESS_BACK"] = get_float(sheet["g{}".format(find_idx(tests, "B/FS_PLAN")[0])], '/')
    metrology_test["results"]["PARALLELISM_BACK"] = get_float(sheet["g{}".format(find_idx(tests, "B/PARAL")[0])])

    # Get defects
    check_for_problems(sheet, metrology_test, [tests.index("WEIGH")+1, sheet.max_row])

    return vi_test, delamination_test, grounding_test, metrology_test, batch, petal_weight


def readProductionSheet(session, fnam, SN):
    """Read data fro AVS PS.

    Args:
    ----
        session: the DB session
        fnam: path of input file
        SN: The serial number
        write_json: if true, test json is writen to file.

    """
    try:
        wb = XL.load_workbook(fnam)
    except InvalidFileException as ee:
        print("Could not open input file: ", fnam)
        print(ee.message)
        return None

    # Assume active sheet is the good one, otherwise will have tofind in wb.sheetnames
    sheet = wb.active
    if sheet.max_row > 30 or sheet.max_column > 8:
        raise AVSDataException("Wrong PS file")

    ID = sheet['c12'].value.strip()
    SN = get_coreID(ID)
    mould_id = sheet['b9'].value
    txt = list(map(str.strip, sheet['f4'].value.split(':')))[1]
    try:
        test_date = dateutil.parser.parse(txt)
    except Exception:
        test_date = ITkDButils.get_db_date(sheet['e8'].value)

    manager = sheet['e7'].value

    # Manufacturing
    comments = get_comments(sheet['a25'].value)
    manufacturing = create_manufacturing(session, SN, test_date, manager, comments=comments)
    manufacturing['properties']['START_DATE'] = ITkDButils.get_db_date(sheet['e8'].value)
    manufacturing['properties']['FINISH_DATE'] = ITkDButils.get_db_date(sheet['e9'].value)
    manufacturing["properties"]["MOULD_ID"] = mould_id
    manufacturing["properties"]["PROCESS_DOCUMENT"] = sheet['e6'].value
    manufacturing["results"]["LOCATOR_A"] = sheet['c14'].value
    manufacturing["results"]["LOCATOR_B"] = sheet['c15'].value
    manufacturing["results"]["LOCATOR_C"] = sheet['c16'].value
    manufacturing["results"]["HONEYCOMBSET"] = split_comp_list(sheet['c17'].value)
    manufacturing["results"]["EPOXY_ADHESIVE"] = split_comp_list(sheet['c20'].value)
    manufacturing["results"]["EPOXY_PUTTY"] = split_comp_list(sheet['c21'].value)
    manufacturing["results"]["EPOXY_CONDUCTIVE"] = split_comp_list(sheet['c22'].value)

    # Weighing
    weighing = create_weight(session, SN, test_date, manager)
    comp_weight = [get_float(x[0]) for x in sheet['d12:d22']]
    petal_weight = sum([float(x) for x in comp_weight])
    weighing["results"]["WEIGHT_FACING_FRONT"] = comp_weight[0]
    weighing["results"]["WEIGHT_FACING_BACK"] = comp_weight[1]
    weighing["results"]["WEIGHT_LOCATOR_A"] = comp_weight[2]
    weighing["results"]["WEIGHT_LOCATOR_B"] = comp_weight[3]
    weighing["results"]["WEIGHT_LOCATOR_C"] = comp_weight[4]
    weighing["results"]["WEIGHT_COOLINGLOOPASSEMBLY"] = comp_weight[6]
    weighing["results"]["WEIGHT_HONEYCOMBSET"] = comp_weight[5]
    weighing["results"]["WEIGHT_EPOXYADHESIVE"] = comp_weight[8]
    weighing["results"]["WEIGHT_EPOXYPUTTY"] = comp_weight[9]
    weighing["results"]["WEIGHT_EPOXYCONDUCTIVE"] = comp_weight[10]
    weighing["results"]["WEIGHT_CORE"] = petal_weight

    # Comments
    for i in range(12, 23):
        cell_id = sheet['B{}'.format(i)].value
        comment = sheet['E{}'.format(i)].value
        if comment is not None:
            comment = comment.strip()
            msg = "{}: {}".format(cell_id, comment)
            weighing["comments"].append(msg)

    DESY = {
        "FacingFront": sheet['c12'].value.strip(),
        "FacingBack": sheet['c13'].value.strip(),
        "CoolingLoop": str(sheet['c18'].value),
        "AllcompSet": sheet['c19'].value,
        "HoneyCombSet": manufacturing["results"]["HONEYCOMBSET"]
    }

    return manufacturing, weighing, DESY


if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument('files', nargs='*', help="Input files")
    parser.add_argument("--SN", dest="SN", type=str, default="SNnnn",
                        help="Module serial number")

    options = parser.parse_args()
    if len(options.files) == 0:
        print("I need an input file")
        sys.exit()

    dlg = ITkDBlogin.ITkDBlogin()
    client = dlg.get_client()
    if client is None:
        print("Could not connect to DB with provided credentials.")
        dlg.die()
        sys.exit()

    fnam = Path(options.files[0]).expanduser().resolve()
#    readProductionSheet(fnam, options.SN)
    readFATfile(client, fnam, options.SN)
    dlg.die()



#!/usr/bin/env python3
"""Read from a Google Sheet."""
import os
import re

from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build

# If modifying these scopes, delete the file token.json.
SCOPES = ['https://www.googleapis.com/auth/spreadsheets.readonly']


# Get spreadsheet ID from share link
re_sheet_id = re.compile(r"https://docs.google.com/spreadsheets/d/(?P<ID>\w+)", re.DOTALL)


def get_spreadsheet_service():
    """Return the spreadsheet service."""
    creds = None
    # The file token.json stores the user's access and refresh tokens, and is
    # created automatically when the authorization flow completes for the first
    # time.
    if os.path.exists('token.json'):
        creds = Credentials.from_authorized_user_file('token.json', SCOPES)
    # If there are no (valid) credentials available, let the user log in.
    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(
                '/Users/lacasta/cl3drv-credentials.json', SCOPES)
            creds = flow.run_local_server(port=0)
        # Save the credentials for the next run
        with open('token.json', 'w') as token:
            token.write(creds.to_json())

    service = build('sheets', 'v4', credentials=creds)
    return service


def get_sheet_id(url):
    """Get sheet ID from google sheet url."""
    M = re.search(re_sheet_id, url)
    if M is None:
        print("Wrong url")
        return

    spread_sheet_id = M.group('ID')
    return spread_sheet_id


def get_spreadsheet_data(url, data_range):
    """Get the data from the given range in teh google sheet.

    Args:
    ----
        url: google sheet document url
        data_range: the data range, ej. inventory!A2:Z

    """
    service = get_spreadsheet_service()

    spread_sheet_id = get_sheet_id(url)

    # Call the Sheets API
    sheet = service.spreadsheets()
    result = sheet.values().get(spreadsheetId=spread_sheet_id,
                                range=data_range).execute()
    values = result.get('values', [])
    return values


if __name__ == "__main__":
    import json
    tb = get_spreadsheet_data("https://docs.google.com/spreadsheets/d/10_CcIKD8IQ69uRLuITuzqSx4oETc1e0ugynOPlruFTE/edit#gid=97165881", "R0 modules")
#    tb = get_spreadsheet_data("https://docs.google.com/spreadsheets/d/1_h4jXYvFp77ax2YceoDx50fQ9ufzK85aTRkqEfmIKgo/edit#gid=0", 'inventory!A189:Z')
    print(json.dumps(tb, indent=2))



#!/usr/bin/env python3
"""Un trash a trashed component."""


def untrash_component(client, SN, reason="Accidentally trashed"):
    """Un trash given component.

    Args:
        SN (str): Serial number of component to recover.
        reason (str): message for the DB
    Return:
        dict: PDB response.

    """
    DTO = {
        'component': SN,
        'trashed': False,
        'reason': reason
    }

    response = client.post('setComponentTrashed', json=DTO)
    return response


if __name__ == "__main__":
    import sys
    from itkdb_gtk import ITkDBlogin
    dlg = ITkDBlogin.ITkDBlogin()
    client = dlg.get_client()

    try:
        response = untrash_component(client, sys.argv[1])

    except Exception as E:
        print(str(E))



#!/usr/bin/env python3
"""Test dashboard."""
import sys

try:
    import dbGtkUtils
    import getShipments
    import groundVITests
    import ITkDBlogin
    import sendShipments
    import uploadMultipleTests
    import GlueWeight
    import uploadIVfiles

except Exception:
    from itkdb_gtk import dbGtkUtils, getShipments, groundVITests, uploadIVfiles
    from itkdb_gtk import ITkDBlogin, sendShipments, uploadMultipleTests, GlueWeight

import gi

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk


class DashWindow(dbGtkUtils.ITkDBWindow):
    """Dashboard class."""
    UPLOAD_TEST = 1 
    CREATE_SHIPMNT = 2
    RECV_SHIPMNT = 3
    PETAL_GND = 4
    GLUE_WEIGHT = 5
    MOD_IV = 6
    
    def __init__(self, session):
        """Initialization."""
        super().__init__(title="ITkDB Dashboard", session=session)
        self.mask = 0

        # set border width
        self.set_border_width(10)

        # Prepare dashboard
        lbl = Gtk.Label()
        lbl.set_markup("<big><b>ITkDB available commands.</b></big>")
        self.mainBox.pack_start(lbl, True, True, 0)

        grid = Gtk.Grid(column_spacing=5, row_spacing=5)
        self.mainBox.pack_start(grid, False, True, 5)

        irow = 0
        lbl = Gtk.Label()
        lbl.set_markup("<b>Tests</b>")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, irow, 1, 1)

        irow += 1
        btnTest = Gtk.Button(label="Upload Tests (json)")
        btnTest.connect("clicked", self.upload_test)
        grid.attach(btnTest, 0, irow, 1, 1)

        btnGnd = Gtk.Button(label="Petal VI/GND")
        btnGnd.connect("clicked", self.petal_gnd)
        grid.attach(btnGnd, 1, irow, 1, 1)

        irow += 1
        btnWeight = Gtk.Button(label="GlueWeight")
        btnWeight.connect("clicked", self.glue_weight)
        grid.attach(btnWeight, 0, irow, 1, 1)

        btnModIV = Gtk.Button(label="Module IV")
        btnModIV.connect("clicked", self.module_IV)
        grid.attach(btnModIV, 1, irow, 1, 1)


        irow += 1
        grid.attach(Gtk.Label(), 0, irow, 1, 1)

        irow += 1
        lbl = Gtk.Label()
        lbl.set_markup("<b>Shipments</b>")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, irow, 1, 1)

        irow += 1
        sendS = Gtk.Button(label="Create Shipment")
        sendS.connect("clicked", self.create_shipment)
        grid.attach(sendS, 0, irow, 1, 1)

        recS = Gtk.Button(label="Receive Shipment")
        recS.connect("clicked", self.receive_shipment)
        grid.attach(recS, 1, irow, 1, 1,)

        self.mainBox.pack_start(Gtk.Separator(orientation=Gtk.Orientation.HORIZONTAL), False, True, 5)

        self.show_all()

    def upload_test(self, *args):
        """Launch upload test."""
        bitn = DashWindow.UPLOAD_TEST
        bt = 1 << bitn
        if self.mask & bt:
            return

        self.mask |= bt
        W = uploadMultipleTests.UploadMultipleTests(self.session)
        W.connect("destroy", self.app_closed, bitn)

    def create_shipment(self, *args):
        """Launch sendShipment."""
        bitn = DashWindow.CREATE_SHIPMNT
        bt = 1 << bitn
        if self.mask & bt:
            return

        self.mask |= bt
        W = sendShipments.SendShipments(self.session)
        W.connect("destroy", self.app_closed, bitn)

    def receive_shipment(self, *args):
        """Launch getShipments."""
        bitn = DashWindow.RECV_SHIPMNT
        bt = 1 << bitn
        if self.mask & bt:
            return

        self.mask |= bt
        W = getShipments.ReceiveShipments(self.session)
        W.connect("destroy", self.app_closed, bitn)

    def petal_gnd(self, *args):
        """Petal GND/VI test."""
        bitn = DashWindow.PETAL_GND
        bt = 1 << bitn
        if self.mask & bt:
            return

        self.mask |= bt
        W = groundVITests.GroundingTest(self.session)
        W.connect("destroy", self.app_closed, bitn)

    def glue_weight(self, *args):
        """Glue Weight test."""
        bitn = DashWindow.GLUE_WEIGHT
        bt = 1 << bitn
        if self.mask & bt:
            return

        self.mask |= bt
        W = GlueWeight.GlueWeight(self.session)
        W.connect("destroy", self.app_closed, bitn)

    def module_IV(self, *args):
        """Module IV tests."""
        bitn = DashWindow.MOD_IV
        bt = 1 << bitn
        if self.mask & bt:
            return

        self.mask |= bt
        W = uploadIVfiles.IVwindow(self.session)
        W.connect("destroy", self.app_closed, bitn)

    def app_closed(self, *args):
        """Application window closed. Clear mask."""
        bt = 1 << args[1]
        self.mask &= ~bt
        print(bt, self.mask)


def main():
    # DB login
    dlg = ITkDBlogin.ITkDBlogin()
    client = dlg.get_client()
    if client is None:
        print("Could not connect to DB with provided credentials.")
        dlg.die()
        sys.exit()

    client.user_gui = dlg

    dashW = DashWindow(client)
    dashW.connect("destroy", Gtk.main_quit)
    try:
        Gtk.main()

    except KeyboardInterrupt:
        print("Arrrgggg!!!")

    dlg.die()


if __name__ == "__main__":
    main()



__version__ = "0.0.16.dev1"


def dash_board():
    """Launches the dash board."""
    from .dashBoard import main
    dashBoard.main()


def getShipments():
    """getShipments."""
    from .getShipments import main
    main()


def glueWeight():
    """glue weight."""
    from .GlueWeight import main
    main()


def groundVITests():
    """GND/VI tests."""
    from .groundVITests import main
    main()


def sendShipments():
    """Send items."""
    from .sendShipments import main
    main()


def uploadTest():
    """Upload tests."""
    from .uploadTest import main
    main()


def uploadMultipleTests():
    """Upload multiple tests."""
    from .uploadMultipleTests import main
    main()

def uploadIVfiles():
    """Upload IV files of single and double modules"""
    from .uploadIVfiles import main
    main()



#!/usr/bin/env python3
"""Uploads data from AVS pdf."""
import json
import sys
from argparse import ArgumentParser
from datetime import datetime
from pathlib import Path

import dateutil.parser
import gi

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio

try:
    import readAVSdata
    import ITkDBlogin
    import ITkDButils

    from dbGtkUtils import replace_in_container, complain, DictDialog, ask_for_confirmation

except ModuleNotFoundError:
    from itkdb_gtk import readAVSdata, ITkDBlogin, ITkDButils
    from itkdb_gtk.dbGtkUtils import replace_in_container, complain, DictDialog, ask_for_confirmation


def create_scrolled_dictdialog(the_dict, hidden=("component", "testType")):
    """Create a DictDialog within a scrolled window.

    Return:
    ------
        scrolled: the scrolled window
        gM: the DictDialog

    """
    gM = DictDialog(the_dict, hidden)
    scrolled = Gtk.ScrolledWindow()
    scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
    scrolled.add(gM)
    return scrolled, gM


def get_type(child):
    """Return object type

    Args:
    -----
        child (): object

    Returns
    -------
        str: object type

    """
    if child["type"] is not None:
        ctype = child["type"]["code"]
    else:
        ctype = child["componentType"]["code"]

    return ctype


class AVSPanel(Gtk.Window):
    """Dialog for interaction with DB."""

    def __init__(self, session, options):
        """Initialization."""
        super().__init__(title="Upload AVS Data")
        self.db_session = session
        self.test_uploaded = {}
        self.test_list = []
        self.test_index = {}
        self.test_panel = {}
        self.test_map = {}
        self.petal_core = None
        self.petal_weight = -1
        # self.set_position(Gtk.WindowPosition.CENTER_ALWAYS)

        #
        # Prepare HeaderBar
        hb = Gtk.HeaderBar()
        hb.set_show_close_button(True)
        hb.props.title = "DB Upload Petal Data"
        self.set_titlebar(hb)

        button = Gtk.Button()
        icon = Gio.ThemedIcon(name="document-send-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        button.add(image)
        button.set_tooltip_text("Click to upload test shown in notebook.")
        button.connect("clicked", self.upload_current_test)
        hb.pack_end(button)

        button = Gtk.Button()
        icon = Gio.ThemedIcon(name="emblem-documents-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        button.add(image)
        button.set_tooltip_text("Click to upload AVS files.")
        button.connect("clicked", self.upload_avs_files)
        hb.pack_end(button)

        button = Gtk.Button()
        icon = Gio.ThemedIcon(name="system-search-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        button.add(image)
        button.set_tooltip_text("Click to search SN in data base.")
        button.connect("clicked", self.query_db)
        hb.pack_end(button)

        self.userLabel = Gtk.Label()
        self.userLabel.set_text(session.user.name)
        hb.pack_start(self.userLabel)

        #
        # Crete man contentt box
        self.mainBox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        self.add(self.mainBox)

        # PS file entry and search button
        self.btnPSF = Gtk.FileChooserButton()
        self.btnPSF.connect("file-set", self.on_psf_set)
        if options.PS:
            ifile = Path(options.PS).expanduser().resolve().as_posix()
            self.btnPSF.set_filename(ifile)
            self.on_psf_set()

        # FAT file entry and seach buttin
        self.btnFAT = Gtk.FileChooserButton()
        self.btnFAT.connect("file-set", self.on_fat_set)
        if options.FAT:
            ifile = Path(options.FAT).expanduser().resolve().as_posix()
            self.btnFAT.set_filename(ifile)

        # The Serial number
        self.SN = Gtk.Entry()
        # self.SN.connect("changed", self.SN_changed)
        if options.SN:
            self.SN.set_text(options.SN)

        # Put the 3 objects in a Grid
        grid = Gtk.Grid(column_spacing=5, row_spacing=1)
        self.mainBox.pack_start(grid, False, True, 0)

        grid.attach(Gtk.Label(label="Serial No."), 0, 0, 1, 1)
        grid.attach(self.SN, 1, 0, 1, 1)

        grid.attach(Gtk.Label(label="Prod. Sheet"), 0, 1, 1, 1)
        grid.attach(self.btnPSF, 1, 1, 1, 1)

        grid.attach(Gtk.Label(label="FAT file"), 0, 2, 1, 1)
        grid.attach(self.btnFAT, 1, 2, 1, 1)

        # Add a Separator
        self.mainBox.pack_start(Gtk.Separator(orientation=Gtk.Orientation.VERTICAL), False, True, 0)

        # The notebook
        self.notebook = Gtk.Notebook()
        self.notebook.set_tab_pos(Gtk.PositionType.LEFT)
        self.mainBox.pack_start(self.notebook, True, True, 20)

        # Create the Notebook pages
        defaults = {
            "institution": "AVS",
            "runNumber": 1,
        }

        self.manufacture = self.create_test_window(
            ITkDButils.get_test_skeleton(
                self.db_session, "CORE_PETAL", "MANUFACTURING", defaults),
            "manufacture", "Manufacture")

        self.weights = self.create_test_window(
            ITkDButils.get_test_skeleton(
                self.db_session, "CORE_PETAL", "WEIGHING", defaults),
            "weights", "Weights")

        self.delamination = self.create_test_window(
            ITkDButils.get_test_skeleton(
                self.db_session, "CORE_PETAL", "DELAMINATION", defaults),
            "delamination", "Delamination")

        self.grounding = self.create_test_window(
            ITkDButils.get_test_skeleton(
                self.db_session, "CORE_PETAL", "GROUNDING_CHECK", defaults),
            "grounding", "Grounding")

        self.metrology = self.create_test_window(
            ITkDButils.get_test_skeleton(
                self.db_session, "CORE_PETAL", "METROLOGY_AVS", defaults),
            "metrology", "Metrology")

        self.visual_inspection = self.create_test_window(
            ITkDButils.get_test_skeleton(
                self.db_session, "CORE_PETAL", "VISUAL_INSPECTION", defaults),
            "visual_inspection", "Visual Inspection")

        # List componentes
        self.components = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        self.components.set_border_width(5)
        self.notebook.append_page(self.components, Gtk.Label(label="Components"))

        # The button box
        btnBox = Gtk.ButtonBox(orientation=Gtk.Orientation.HORIZONTAL)

        btn = Gtk.Button(label="Reload AVS files")
        btn.connect("clicked", self.read_avs_files)
        btnBox.add(btn)

        btn = Gtk.Button(label="Query DB")
        btn.connect("clicked", self.query_db)
        btnBox.add(btn)

        btn = Gtk.Button(label="Assemble")
        btn.connect("clicked", self.on_assembly)
        btnBox.add(btn)

        btn = Gtk.Button(label="Upload Tests")
        btn.connect("clicked", self.on_upload)
        btnBox.add(btn)

        btn = Gtk.Button(label="Quit")
        btn.connect("clicked", Gtk.main_quit)
        btnBox.add(btn)

        self.mainBox.pack_start(btnBox, False, True, 0)
        self.connect("destroy", Gtk.main_quit)

    def create_test_window(self, test_json, test_name, label):
        """Create the dialog for a DB test and add it to the notebook.

        Args:
        ----
            test_json: The JSon-like dict with the values
            test_name: The name of the test for internal indexing
            label: The label for the Notebook

        Return:
        ------
            The box containing the data.

        """
        scrolled, gM = create_scrolled_dictdialog(test_json)
        self.test_list.append(gM)
        self.test_index[test_name] = len(self.test_list) - 1
        self.test_map[test_json["testType"]] = test_name

        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        box.set_border_width(5)
        box.pack_end(scrolled, True, True, 0)
        box.dict_dialog = gM
        gM.box = box
        self.test_panel[test_name] = box
        self.notebook.append_page(box, Gtk.Label(label=label))

        return box

    def update_scroll_window(self, key, data):
        """Update panel for a given test."""
        scrolled, gM = create_scrolled_dictdialog(data, ("component", "testType"))
        self.test_list[self.test_index[key]] = gM
        replace_in_container(self.test_panel[key], scrolled)

    def check_register_petal(self, SN):
        """Register petal core in DB.

        Args:
        ----
            SN: The petal Serial Number.

        """
        if self.petal_core:
            return

        self.find_petal(SN, silent=True)
        if self.petal_core:
            return

        dialog = Gtk.MessageDialog(
            transient_for=self,
            flags=0,
            message_type=Gtk.MessageType.INFO,
            text="Register Petal Core\n{}".format(SN)
        )
        dialog.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                           Gtk.STOCK_OK, Gtk.ResponseType.OK)
        dialog.set_border_width(10)
        dialog.format_secondary_text("Enter Petal Core Alias")
        alias = Gtk.Entry()
        box = dialog.get_content_area()
        box.add(alias)
        dialog.show_all()
        out = dialog.run()
        if out == Gtk.ResponseType.OK:
            petal_alias = alias.get_text()
            rc = ITkDButils.registerPetalCore(self.db_session, SN, petal_alias)
            if rc is None:
                complain("Could not Register petal {} ({})".format(SN, petal_alias))

        dialog.hide()
        dialog.destroy()

    def get_app_petal_serial_number(self):
        """Get the SN from the text box."""
        txt_SN = self.SN.get_text().strip()
        return txt_SN

    def check_petal_serial_number(self, SN):
        """Check tha the given SN is consistent."""
        txt_SN = self.get_app_petal_serial_number()
        if txt_SN and len(txt_SN):
            if txt_SN != SN.strip():
                return False

            else:
                return True

        else:
            self.check_register_petal(SN)
            self.SN.set_text(SN)
            return True

    def on_psf_set(self, *args):
        """Production Sheet file selected."""
        PSF = self.btnPSF.get_filename()
        if PSF is None or not Path(PSF).exists():
            complain("Could not find Production File", PSF, parent=self)
            return

        try:
            manuf_json, weights_json, self.DESY_comp = readAVSdata.readProductionSheet(self.db_session, PSF, "SNnnnn")
            if self.petal_weight > 0:
                weights_json["results"]["WEIGHT_CORE"] = self.petal_weight

        except readAVSdata.AVSDataException as E:
            complain("Wrong Production Sheet file", str(E))
            self.btnPSF.unselect_all()
            return

        SN = manuf_json["component"]
        if not self.check_petal_serial_number(SN):
            complain("Inconsistent Serial number found.",
                     "Wrong Serial number extracted from the Production Sheet document.\n{}".format(PSF))
            self.btnPSF.unselect_all()
            return

        scrolled, gM = create_scrolled_dictdialog(manuf_json, ("component", "testType"))
        self.test_list[self.test_index["manufacture"]] = gM
        replace_in_container(self.manufacture, scrolled)

        scrolled, gM = create_scrolled_dictdialog(weights_json, ("component", "testType"))
        self.test_list[self.test_index["weights"]] = gM
        replace_in_container(self.weights, scrolled)

        gD = DictDialog(self.DESY_comp)
        replace_in_container(self.components, gD)

        # Check if we need to assemble the module
        self.check_assembly(self.DESY_comp)

    def on_fat_set(self, *args):
        """FAT file selected."""
        FAT = self.btnFAT.get_filename()
        if FAT is None or not Path(FAT).exists():
            complain("Could not find FAT File", FAT, parent=self)

        try:
            SN = self.get_app_petal_serial_number()
            if SN and not SN.startswith("20USEBC"):
                SN = None

            j_vi, j_del, j_gnd, j_mtr, batch, self.petal_weight = readAVSdata.readFATfile(self.db_session, FAT, SN)
            self.test_list[self.test_index["weights"]].set_value("results.WEIGHT_CORE", self.petal_weight)

        except readAVSdata.AVSDataException as E:
            complain("Wrong FAT file", str(E))
            self.btnFAT.unselect_all()
            return

        SN = j_vi["component"]
        if not self.check_petal_serial_number(SN):
            complain("Inconsistent Serial number found.",
                     "Wrong Serial number extracted from the FAT document.\n{}".format(FAT))
            self.btnFAT.unselect_all()
            return

        self.update_scroll_window("visual_inspection", j_vi)
        self.update_scroll_window("delamination", j_del)
        self.update_scroll_window("grounding", j_gnd)
        self.update_scroll_window("metrology", j_mtr)

    def read_avs_files(self, widgets):
        """Read AVS files."""
        PSF = self.btnPSF.get_filename()
        if PSF is not None:
            self.on_psf_set(None)

        FAT = self.btnFAT.get_filename()
        if FAT is not None:
            self.on_fat_set(None)

        return

    def find_petal(self, SN, silent=False):
        """Finds petal with given SN."""
        try:
            self.petal_core = ITkDButils.get_DB_component(self.db_session, SN)

        except Exception as E:
            if not silent:
                complain("Could not find Petal Core in DB", str(E))

            self.petal_core = None
            return

        try:
            if self.petal_core["type"]["code"] != "CORE_AVS":
                complain("Wrong component type", "This is not an AVS petal core")

            if self.petal_core["currentStage"]["code"] != "ASSEMBLY":
                complain("Wrong component stage", "Wrong stage: {}".format(self.petal_core["currentStage"]["code"]))

            print(json.dumps(self.petal_core, indent=3))

        except KeyError:
            # Petal is not there
            self.petal_core = None

    def query_db(self, widget=None, silent=False):
        """Called when QueryDB button clicked."""
        SN = self.SN.get_text()
        if len(SN) == 0:
            complain("Empty Serial number",
                     "You should enter a valid Serial number for the petal core.",
                     parent=self)
            self.petal_core = None
            return

        # if not checkSerialNumber(SN):
        #     complain("Wrong Serial number",
        #              "You should enter a valid Serial number for the petal core.",
        #              parent=self)
        #     return
        self.find_petal(SN, silent=silent)

        if self.btnFAT.get_filename() is None and self.btnPSF.get_filename() is None:
            # update tests from DB
            for test in self.petal_core["tests"]:
                latest = None
                latest_time = datetime.datetime(year=1, month=1, day=1, tzinfo=datetime.timezone.utc)
                testType = test["code"]
                for T in test["testRuns"]:
                    test_date = dateutil.parser.parse(T["cts"])
                    if test_date > latest_time:
                        latest_time = test_date
                        latest = T["id"]

                dbT = ITkDButils.get_testrun(self.db_session, latest)
                testDB = ITkDButils.from_full_test_to_test_data(dbT)
                self.update_scroll_window(self.test_map[testType], testDB)

    def check_assembly(self, components):
        """Check if we need to assemble components to core."""
        if self.petal_core is None:
            self.query_db()
            if self.petal_core is None:
                return

        comp_map = {
            "BT_PETAL_FRONT": "FacingFront",
            "BT_PETAL_BACK": "FacingBack",
            "COOLING_LOOP_PETAL": "CoolingLoop",
            "THERMALFOAMSET_PETAL": "AllcompSet"
        }
        final_stage = {
            "BT_PETAL_FRONT": "COMPLETED",
            "BT_PETAL_BACK": "COMPLETED",
            "COOLING_LOOP_PETAL": "CLINCORE",
            "THERMALFOAMSET_PETAL": "IN_CORE"
        }
        missing = []
        for child in self.petal_core["children"]:
            if child["component"] is None:
                if child["type"] is not None:
                    ctype = child["type"]["code"]
                else:
                    ctype = child["componentType"]["code"]

                missing.append(ctype)

        if len(missing) == 0:
            return

        error_txt = []
        txt = "Click OK to add\n\t{}".format("\n\t".join(missing))
        if ask_for_confirmation("Missing components", txt, parent=self):
            this_petal = self.SN.get_text()
            for cmp in missing:
                SN = components[comp_map[cmp]]
                if SN[0:5] == "20USE":
                    rc = ITkDButils.assemble_component(self.db_session, this_petal, SN)
                    if rc is None:
                        error_txt.append("Problem assembling {} into Petal\n".format(cmp))

            # Check for HonneyComb set
            for P in self.petal_core["properties"]:
                if P["code"] == "HC_ID" and P["value"] is None:
                    rc = ITkDButils.set_component_property(self.db_session,
                                                           this_petal,
                                                           "HC_ID",
                                                           components["HoneyCombSet"])
                    if rc is None:
                        error_txt.append("Problems setting HoneCombSet ID.\n")

                    break

            # Check the final stage of the assembled objects
            for child in self.petal_core["children"]:
                if child["component"]:
                    cSN = child["component"]["serialNumber"]
                    ctype = get_type(child)
                    cobj = ITkDButils.get_DB_component(session, cSN)
                    cstage = cobj["currentStage"]['code']
                    if cstage != final_stage[ctype]:
                        rc = ITkDButils.set_object_stage(session, cSN, final_stage[ctype])
                        if rc is None:
                            print("Could not set final stage of {}".format(cSN))

            if len(error_txt):
                complain("Assembly of {} could not be completeed:".format(this_petal),
                         "\n".join(error_txt))

    def upload_current_test(self, *args):
        """Called with upload button clcked."""
        SN = self.SN.get_text()
        if len(SN) == 0:
            complain("Petal SN is empty")
            return

        def find_children(W):
            try:
                for c in W.get_children():
                    if "DictDialog" in c.get_name():
                        return c

                    else:
                        return find_children(c)

            except Exception:
                return None

            return None

        page = self.notebook.get_nth_page(self.notebook.get_current_page())
        dctD = find_children(page)
        if dctD is None:
            return

        values = dctD.values
        values["component"] = SN
        print(json.dumps(values, indent=2))
        rc = ITkDButils.upload_test(self.db_session, values)
        if rc is not None:
            complain("Could not upload test", rc)

        else:
            ask_for_confirmation("Test uploaded.",
                                 "{} - {}".format(values["component"], values["testType"]))

    def upload_avs_files(self, *args):
        """Called when upload AVS files clicked."""
        SN = self.SN.get_text()
        if len(SN) == 0:
            complain("Petal SN is empty")
            return

        def upload_file(file_path, title, desc):
            if file_path is not None:
                if not Path(file_path).exists():
                    complain("Could not find {}".format(title))

                else:
                    try:
                        ITkDButils.create_component_attachment(self.db_session, SN, file_path, description=desc)

                    except Exception as e:
                        complain("Could not Upload {}".format(desc), str(e))

        PSF = self.btnPSF.get_filename()
        upload_file(PSF, "Production File", "AVS Production file")

        FAT = self.btnFAT.get_filename()
        upload_file(FAT, "FAT file", "AVS FAT file")

    def on_assembly(self, widget):
        """Assembly button clicked."""
        self.check_assembly(self.DESY_comp)

    def on_upload(self, widget):
        """Upload tests to DB."""
        if self.petal_core is None:
            self.query_db()
            if self.petal_core is None:
                return

        for test in self.test_list:
            values = test.values
            print(values["testType"])
            res = ITkDButils.upload_test(self.db_session, values)
            if res is not None:
                complain("Could not upload test {}".format(values["testType"]), res)


if __name__ == "__main__":
    # Parse command line options
    parser = ArgumentParser()
    parser.add_argument('files', nargs='*', help="Input files")
    parser.add_argument("--SN", dest="SN", type=str, default=None,
                        help="Module serial number")
    parser.add_argument("--PS", dest="PS", type=str, default=None,
                        help="Produc tion Sheet file")
    parser.add_argument("--FAT", dest="FAT", type=str, default=None,
                        help="FAT file")

    options = parser.parse_args()

    # ITk_PB authentication
    dlg = ITkDBlogin.ITkDBlogin()
    session = dlg.get_client()

    # Start the Application
    win = AVSPanel(session, options)
    win.show_all()
    win.set_accept_focus(True)
    win.present()
    try:
        Gtk.main()
    except KeyboardInterrupt:
        print("Arrggggg !!!")

    print("Bye !!")
    dlg.die()
    sys.exit()



from pathlib import Path
try:
    import dbGtkUtils
    import ITkDButils
except ModuleNotFoundError:
    from itkdb_gtk import dbGtkUtils, ITkDButils

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio, Gdk


def add_attachment_dialog():
    """Create the add attachment dialog."""
    dlg = Gtk.Dialog(title="Add Attachment")
    dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                    Gtk.STOCK_OK, Gtk.ResponseType.OK)
    grid = Gtk.Grid(column_spacing=5, row_spacing=1)
    box = dlg.get_content_area()
    box.add(grid)

    lbl = Gtk.Label(label="File")
    lbl.set_xalign(0)
    grid.attach(lbl, 0, 0, 1, 1)

    lbl = Gtk.Label(label="Title")
    lbl.set_xalign(0)
    grid.attach(lbl, 0, 1, 1, 1)

    lbl = Gtk.Label(label="Description")
    lbl.set_xalign(0)
    grid.attach(lbl, 0, 2, 1, 1)

    dlg.fC = Gtk.FileChooserButton()
    grid.attach(dlg.fC, 1, 0, 1, 1)

    dlg.att_title = Gtk.Entry()
    grid.attach(dlg.att_title, 1, 1, 1, 1)

    dlg.att_desc = Gtk.Entry()
    grid.attach(dlg.att_desc, 1, 2, 1, 1)

    dlg.show_all()
    return dlg


class ShowAttachments(Gtk.Dialog):
    """Window to show attachments."""
    def __init__(self, title, session, attachments=[], parent=None):
        """Initialization."""
        super().__init__(title=title, transient_for=parent)
        self.session = session
        self.attachments = [A for A in attachments]
        self.init_window()

    def init_window(self):
        """Prepares the window."""
        self.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                         Gtk.STOCK_OK, Gtk.ResponseType.OK)

        self.mainBox = self.get_content_area()
        # The "Add attachment" button.
        box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        self.mainBox.pack_start(box, False, False, 0)

        dbGtkUtils.add_button_to_container(box, "Add attachment",
                                           "Click to add a new attachment.",
                                           self.add_attachment)

        dbGtkUtils.add_button_to_container(box, "Remove attachment",
                                           "Click to remove selected attachment.",
                                           self.remove_attachment)

        # the list of attachments
        tree_view = self.create_tree_view()
        self.mainBox.pack_start(tree_view, True, True, 0)
        for A in self.attachments:
            self.append_attachment_to_view(A)

        self.show_all()

    def create_tree_view(self, size=150):
        """Creates the tree vew with the attachments."""
        model = Gtk.ListStore(str, str, str, str)
        self.tree = Gtk.TreeView(model=model)
        self.tree.connect("button-press-event", self.button_pressed)

        scrolled = Gtk.ScrolledWindow()
        scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        scrolled.add(self.tree)
        scrolled.set_size_request(-1, size)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Attachment", renderer, text=0)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Title", renderer, text=1)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Description", renderer, text=2)
        self.tree.append_column(column)

        return scrolled

    def button_pressed(self, tree, event):
        """Button clicked on top of attachment."""
        # double click shows attachments
        it = None
        model = self.tree.get_model()
        if event.button == 1 and event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS:
            select = self.tree.get_selection()
            model, it = select.get_selected()
            if not it:
                return

            self.edit_attachment(model, it)
            return

        if event.button != 3:
            return

        # Create popup menu
        select = self.tree.get_selection()
        model, it = select.get_selected()
        values = None
        if it:
            values = model[it]

        else:
            P = tree.get_path_at_pos(event.x, event.y)
            if P:
                it = model.get_iter(P[0])
                values = model[it]

        if not values:
            return

        if not it:
            P = self.tree.get_path_at_pos(event.x, event.y)
            if P:
                it = model.get_iter(P[0])

        if not it:
            return

        menu = Gtk.Menu()
        item_edit = Gtk.MenuItem(label="Edit")
        item_edit.connect("activate", self.on_edit_attachment, (model, it))
        menu.append(item_edit)

        item_show = Gtk.MenuItem(label="Show file")
        item_show.connect("activate", self.on_show_file, (model, it))
        menu.append(item_show)

        menu.show_all()
        menu.popup_at_pointer(event)

    def edit_attachment(self, model, it):
        """Edit attachment at current row."""
        values = model[it]
        dlg = add_attachment_dialog()
        dlg.fC.set_filename(values[3])
        dlg.att_title.set_text(values[1])
        dlg.att_desc.set_text(values[2])
        rc = dlg.run()
        if rc == Gtk.ResponseType.OK:
            path = Path(dlg.fC.get_filename())
            title = dlg.att_title.get_text()
            desc = dlg.att_desc.get_text()
            model.set_value(it, 0, path.name)
            model.set_value(it, 1, title)
            model.set_value(it, 2, desc)
            model.set_value(it, 3, path.as_posix())

        dlg.hide()
        dlg.destroy()

    def on_edit_attachment(self, item, data):
        """Test JSon."""
        model, it = data
        self.edit_attachment(model, it)


    def on_show_file(self, item, data):
        """Test JSon."""
        model, it  = data
        values = model[it]
        sss = None
        try:
            with open(values[3], "tr", encoding="utf-8") as f:
                sss = f.read()
        
        except UnicodeDecodeError:
            dbGtkUtils.complain("Error showing file", "File is not a text file")
            return        
        
        
        dlg = Gtk.Dialog(title="Add Attachment",
                     transient_for=self,
                     flags=0)
        dlg.add_buttons(Gtk.STOCK_OK, Gtk.ResponseType.OK)
        dlg.set_size_request(300, 200)
        area = dlg.get_content_area()
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        area.add(box)

        scrolledwindow = Gtk.ScrolledWindow()
        box.pack_start(scrolledwindow, False, True, 0)

        scrolledwindow.set_hexpand(True)
        scrolledwindow.set_vexpand(True)
        textview = Gtk.TextView()
        textview.get_buffer().set_text(sss)
        scrolledwindow.add(textview)
        dlg.show_all()
        dlg.run()
        dlg.hide()
        dlg.destroy()
        

    def add_attachment(self, *args):
        """Add Attachment button clicked."""
        dlg = add_attachment_dialog()
        rc = dlg.run()
        if rc == Gtk.ResponseType.OK:
            path = Path(dlg.fC.get_filename())
            A = ITkDButils.Attachment(path, dlg.att_title.get_text().strip(), dlg.att_desc.get_text().strip())
            self.append_attachment_to_view(A)
            self.attachments.append(A)

        dlg.hide()
        dlg.destroy()

    def append_attachment_to_view(self, A):
        """Insert attachment to tree view."""
        model = self.tree.get_model()
        model.append([A.path.name, A.title, A.desc, A.path.as_posix()])

    def remove_attachment(self, *args):
        """Remove selected attachment."""
        select = self.tree.get_selection()
        model, it = select.get_selected()
        if it:
            values = model[it]
            for a in self.attachments:
                if a.path == values[3]:
                    rc = dbGtkUtils.ask_for_confirmation("Remove this attachment ?",
                                                         "{} - {}\n{}".format(a.title, a.desc, values[0]))
                    if rc:
                        self.attachments.remove(a)
                        model.remove(it)

                    break

if __name__ == "__main__":
    SA = ShowAttachments("Test Attachments", None)
    SA.run()



#!/usr/bin/env python3
"""Read IV files and create plots.

Analisis de la IV con macros de la webApp

SENSOR_IV_Analysis.py in
https://gitlab.cern.ch/atlas-itk/sw/db/production_database_scripts.git

webApp aqui:
https://itk-pdb-webapps-strips.web.cern.ch

"""
import os
import json
import warnings
from pathlib import Path

import gi
import tempfile

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio


from matplotlib.backends.backend_gtk3agg import FigureCanvasGTK3Agg as FigureCanvas
from matplotlib.backends.backend_gtk3 import NavigationToolbar2GTK3 as NavigationToolbar
import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt

try:
    import dbGtkUtils
    import ITkDBlogin
    import ITkDButils
    import uploadTest

except ModuleNotFoundError:
    from itkdb_gtk import dbGtkUtils, ITkDBlogin, ITkDButils, uploadTest

# Check if Gtk can be open
gtk_runs, gtk_args = Gtk.init_check()


def remove_files(W, flist):
    for f in flist:
        os.unlink(f)


#
#   The following is taken from
#   https://gitlab.cern.ch/atlas-itk/sw/db/production_database_scripts.git
#
mm = 1e-3
cm = 1e-2
UpperV = 500  # For sensor to pass, I must be < Imax up until this voltage and no breakdown must be detected before then.
StabilityV = 700  # Voltage with multiple current readings to check stability

AreaDict = {
    "Unknown": (97.621 - 0.450) * (97.950 - 0.550) * mm * mm,
    #
    "ATLAS12": 95.7 * 95.64 * mm * mm,
    "ATLAS17LS": (97.621 - 0.450) * (97.950 - 0.550) * mm * mm,
    #
    "ATLAS18R0": 89.9031 * cm * cm,
    "ATLAS18R1": 89.0575 * cm * cm,
    "ATLAS18R2": 74.1855 * cm * cm,
    "ATLAS18R3": 80.1679 * cm * cm,
    "ATLAS18R4": 87.4507 * cm * cm,
    "ATLAS18R5": 91.1268 * cm * cm,
    #
    "ATLAS18SS": 93.6269 * cm * cm,
    "ATLAS18LS": 93.6269 * cm * cm,
    #
    "ATLASDUMMY18": (97.621 - 0.450) * (97.950 - 0.550) * mm * mm,
}


def LocateMicroDischarge(
    I,
    V,
    sm_window=2,
    bd_limit=5.5,
    allow_running_bd=True,
    use_additional_cond=False,
    tolerence=0.05,
    voltage_span=4,
    fit_window=5,
):
    """
    Function for BDV estimation - if questions please contact Vera Latonova (vera.latonova@cern.ch).
    I,V must have same shape and voltages must be in ascending order,
    same indexes of I&V arrays must correspond each other,
    only invalid data or holdstep should be stripped before
    but it is not necessary. Measurments U=0&I=0 are removed.
    If there is same or higher amount of same voltages in row than
    sm_window, from this sequence we cannot estimete dI/dV and
    we have to remove this averaged point.

    It is assumed that only parameter use_additional_cond would be
    changed by caller. Changing of other parameters may affect
    BDV unexpectedly.


    @param[in] I                   - array of currents without any cut
    @param[in] V                   - array of voltages, ascending order, without any cut
    @param[in] sm_window           - size of smoothing window
    @param[in] bd_limit            - BD limit for |U| < 500V
    @param[in] allow_running_bd    - allow increase bd_limit for |U| > 500
    @param[in] use_additional_cond - use additional BD contition
    @param[in] tolerence           - configuration of additional condition
    @param[in] voltage_span        - max width of hump on spectra which may be neglected
                                     in voltage steps in additional contition
    @param[in] fit_window          - number of points used for linear fit before BD voltage

    @return BD voltage (always positive) or NO_BD_CONST = 9.99e99 if not found.
    """
    NO_BD_CONST = 9.99e99

    # add nan to the end of array
    V = np.abs(V)
    I = np.abs(I)

    # skip zeros
    ind = np.where(np.logical_or(I != 0, V != 0))
    V = V[ind]
    I = I[ind]

    V_ = np.append(V, np.nan * np.ones(sm_window - 1))
    I_ = np.append(I, np.nan * np.ones(sm_window - 1))

    # make 2D array of I's, V's each row_ind shifted by row_ind index
    # i.e from array [1,3,5] we make (for sm_window=2) 2D array
    # [  1,3,5,nan]
    # [nan,5,1,3]
    # than get average from each column -> I_avg, V_avg
    r = np.arange(sm_window)

    V2 = np.outer(np.ones(sm_window), V_)
    row_ind, col_ind = np.ogrid[: V2.shape[0], : V2.shape[1]]
    col_ind = col_ind - r[:, np.newaxis]
    V2 = V2[row_ind, col_ind]
    # strip fields with nans
    V2 = np.transpose(V2[:, (sm_window - 1) : -(sm_window - 1)])

    I2 = np.outer(np.ones(sm_window), I_)
    row_ind, col_ind = np.ogrid[: I2.shape[0], : I2.shape[1]]
    col_ind = col_ind - r[:, np.newaxis]
    I2 = I2[row_ind, col_ind]
    I2 = np.transpose(I2[:, (sm_window - 1) : -(sm_window - 1)])

    # get V & I averages
    try:
        V_avg = np.average(V2, axis=1)
        I_avg = np.average(I2, axis=1)
    except ZeroDivisionError:
        # not enough data
        return NO_BD_CONST

    # find dI / dV array
    # I'm not able to write this without cycle
    dIdV = np.array([])
    for i in range(V2.shape[0]):
        with warnings.catch_warnings():
            warnings.filterwarnings("error")
            try:
                dIdV = np.append(dIdV, np.polyfit(V2[i, :], I2[i, :], 1)[0])
            except (np.RankWarning, TypeError):
                dIdV = np.append(dIdV, np.nan)

    # stripping U[n] == U[n+1] (i.e. hodlsetp) => fit cannot be sucessful =>
    # dIdV is nan @holdstep
    ind = np.where(np.isfinite(dIdV))
    I_avg = I_avg[ind]
    V_avg = V_avg[ind]
    dIdV = dIdV[ind]

    # get running BDV limit & compare
    bd_limit_running = bd_limit + np.where(
        allow_running_bd and V_avg > 500, (V_avg - 500.0) / 100.0, 0
    )
    V_avg_BD_ind = dIdV / (I_avg / V_avg) > bd_limit_running
    V_avg_BD = V_avg[V_avg_BD_ind]

    # Estimate BDV
    BDV = np.array([])

    # no break-down
    if V_avg_BD.shape == (0,):
        return NO_BD_CONST

    # if V_avg_BD_ind[0] == True ... BDV <- V[0]
    # for others V_avg_BD_ind[n] == True BDV <- (V_avg[n] + V_avg[n-1])/2
    if V_avg_BD_ind[0]:
        BDV = np.append(BDV, V[0])
    V_avg_BD_ind[0] = False

    BDV = np.append(
        BDV,
        (V_avg[np.where(V_avg_BD_ind)] + V_avg[np.where(V_avg_BD_ind)[0] - 1]) / 2.0,
    )

    ###########################################################################
    ## Application of additional condition ####################################
    ###########################################################################
    if not use_additional_cond:
        return BDV[0]

    # get index if V <= BDV
    B = np.where(np.less.outer(BDV, V))
    col_ind = np.mgrid[: BDV.shape[0], : V.shape[0]][1]
    col_ind[B[0], B[1]] = 0
    V_BDV_ind = np.max(col_ind, axis=1)

    back_ok_v_ind = 0
    while True:
        with warnings.catch_warnings():
            warnings.filterwarnings("error")
            try:
                a, b = np.polyfit(
                    V[
                        max(back_ok_v_ind, V_BDV_ind[0] - fit_window) : max(
                            back_ok_v_ind, V_BDV_ind[0]
                        )
                    ],
                    I[
                        max(back_ok_v_ind, V_BDV_ind[0] - fit_window) : max(
                            back_ok_v_ind, V_BDV_ind[0]
                        )
                    ],
                    1,
                )
            except (np.RankWarning, TypeError):
                return BDV[0]

        ind = np.where(1 - (a * V + b) / I <= tolerence)[0]
        try:
            back_ok_v_ind = np.min(ind[ind > V_BDV_ind[0] + 1])
        except ValueError:
            # sensor is not going back
            return BDV[0]
        # hump is too long -- it cannot be skipped
        if back_ok_v_ind - V_BDV_ind[0] > voltage_span:
            return BDV[0]

        # skip BDVs inside hump
        ind = BDV >= V[back_ok_v_ind]
        BDV = BDV[ind]
        V_BDV_ind = V_BDV_ind[ind]
        if V_avg_BD.shape == (0,):
            return NO_BD_CONST
    return NO_BD_CONST


def scale_iv(I, T1, T2):
    """Normalize corrent  to given temperature (T2)

    Args:
        I (array): Current
        T1 (float): Original temperature
        T2 (float): New temperature.

    Return:
        Array with scaled currents.

    """
    factor = (T2 / T1) ** 2 * np.exp((-1.21 / 8.62) * (1 / T2 - 1 / T1))
    return factor * I


class IVwindow(dbGtkUtils.ITkDBWindow):
    """GUI for IV file handling."""

    def __init__(self, session, title="IV window", options=None):
        """Initialization."""
        super().__init__(
            session=session, title=title, show_search=None, gtk_runs=gtk_runs
        )
        self.mdata = {}
        self.mod_type = {}
        self.mod_SN = {}
        self.difference = None
        self.canvas = None

        self.init_window()

    def init_window(self):
        """Prepare the Gtk window."""
        self.hb.props.title = "IV data"

        button = Gtk.Button()
        icon = Gio.ThemedIcon(name="view-refresh-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        button.add(image)
        button.set_tooltip_text("Click to refresh canvas.")
        button.connect("clicked", self.on_refresh)
        self.hb.pack_end(button)

        # Button to upload
        button = Gtk.Button()
        icon = Gio.ThemedIcon(name="document-send-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        button.add(image)
        button.set_tooltip_text("Click to upload test")
        button.connect("clicked", self.upload_test)
        self.hb.pack_end(button)

        # File entry and search button
        self.single_file = Gtk.FileChooserButton()
        self.single_file.connect("file-set", self.on_single_file_set)

        self.double_file = Gtk.FileChooserButton()
        self.double_file.connect("file-set", self.on_double_file_set)

        self.single_SN = Gtk.Label(label="(None)")
        self.double_SN = Gtk.Label(label="(None)")

        grid = Gtk.Grid(column_spacing=5, row_spacing=1)

        grid.attach(Gtk.Label(label="Files"), 1, 0, 1, 1)
        grid.attach(Gtk.Label(label="Serial No."), 2, 0, 1, 1)

        grid.attach(Gtk.Label(label="Single Data File"), 0, 1, 1, 1)
        grid.attach(self.single_file, 1, 1, 1, 1)
        grid.attach(self.single_SN, 2, 1, 1, 1)

        grid.attach(Gtk.Label(label="Double Data File"), 0, 2, 1, 1)
        grid.attach(self.double_file, 1, 2, 1, 1)
        grid.attach(self.double_SN, 2, 2, 1, 1)

        btn = Gtk.Button(label="Compute difference")
        btn.connect("clicked", self.on_difference)
        grid.attach(btn, 1, 3, 1, 1)

        btn = Gtk.Button(label="Upload to DB")
        btn.connect("clicked", self.upload_test)
        grid.attach(btn, 2, 3, 1, 1)

        self.mainBox.pack_start(grid, False, True, 0)

        self.fig = mpl.figure.Figure()
        self.fig.tight_layout()
        sw = Gtk.ScrolledWindow()  # Gtk.Box(orientation=Gtk.Orientation.VERTICAL)

        # A scrolled window border goes outside the scrollbars and viewport
        sw.set_border_width(10)
        sw.set_size_request(310, 310)

        self.canvas = FigureCanvas(self.fig)  # a Gtk.DrawingArea
        self.canvas.set_size_request(400, 300)
        sw.add(self.canvas)
        self.mainBox.pack_start(sw, True, True, 0)

        # Create toolbar
        toolbar = NavigationToolbar(self.canvas)
        self.mainBox.pack_start(toolbar, False, False, 0)

        # The text view
        frame = self.create_text_view()
        self.mainBox.pack_start(frame, True, True, 5)

        self.show_all()

    def upload_test(self, *args):
        """Upload available tests."""
        try:
            mdata = self.mdata["double"]
        except KeyError:
            dbGtkUtils.complain("Cannot Upload to DB", "Data not yet ready.")
            return

        # Check if we are dealing with sensors or modules
        is_module = "Module_SN" in mdata

        if is_module:
            test = ITkDButils.get_test_skeleton(
                self.session, "MODULE", self.mdata["double"]["TestType"]
            )
            tp = "ATLAS18{}".format(self.mod_type["double"][0:2])
            area = AreaDict[tp] / cm**2

        else:
            test = ITkDButils.get_test_skeleton(
                self.session, "SENSOR", self.mdata["double"]["TestType"]
            )
            area = AreaDict[self.mod_type["double"]] / cm**2

        # The data arrays
        V = np.abs(mdata["curve"]["V"])
        I = np.abs(mdata["curve"]["I"])
        passed = True

        # Find Current @ 500V
        indx = np.where(V == 500)[0]
        i_500 = I[indx][0] / area
        self.write_message("I @ 500V = {:.2f} nA/cm2\n".format(i_500))

        # Compute current stability
        IStability = abs(I[abs(V) == StabilityV])
        IVariation = -1
        if np.size(IStability) > 1:  # Maybe make == 4?
            IVariation = abs(np.std(IStability) / np.mean(IStability))

        self.write_message("I stability = {:.6f} nA\n".format(IVariation))

        # Search for Micro discharges
        # Check for micro-discharge in non-normalized current,
        # removing duplicate Voltage entries (e.g. for stability measurements)
        comments = []
        defects = []
        UniqueVs, UniqueIndices = np.unique(V, return_index=True)
        MicroDischargeV = LocateMicroDischarge(I[UniqueIndices], UniqueVs)
        if MicroDischargeV < np.max(V):
            comments.append("Found micro discharge: {:.1f} V\n".format(MicroDischargeV))
            self.write_message(comments[-1])

            if MicroDischargeV < UpperV:
                txt = "microdischarge happening before {:.1f}V.".format(UpperV)
                defects.append({
                            "name": "MicroDischarge",
                            "description": txt,
                            "properties": {}
                        }
                )
                self.write_message("...{}. FAILED\n".format(txt))
                passed = False

        test["component"] = self.mod_SN["double"]
        test["institution"] = mdata["Institute"]
        test["runNumber"] = mdata["RunNumber"]
        test["date"] = ITkDButils.get_db_date(
            "{} {}".format(mdata["Date"], mdata["Time"])
        )
        test["passed"] = passed
        test["problems"] = False
        test["properties"]["VBIAS_SMU"] = mdata["Vbias_SMU"]
        test["properties"]["RSERIES"] = mdata["Rseries"]
        test["properties"]["TEST_DMM"] = mdata["Test_DMM"]
        test["properties"]["RSHUNT"] = mdata["Rshunt"]
        test["properties"]["RUNNUMBER"] = mdata["RunNumber"]
        test["properties"]["COMMENTS"] = mdata["Comments"]
        test["properties"]["ALGORITHM_VERSION"] = "0.0.0"
        if is_module:
            test["properties"]["SOFTWARE_TYPE_VERSION"] = "pyProbe"
            test["properties"]["MODULE_STAGE"] = mdata["Module_Stage"]

        test["results"]["TEMPERATURE"] = mdata["Temperature"]
        test["results"]["HUMIDITY"] = mdata["Humidity"]
        test["results"]["VBD"] = MicroDischargeV
        test["results"]["I_500V"] = i_500
        test["results"]["VOLTAGE"] = -np.abs(V)
        test["results"]["CURRENT"] = -np.abs(I)
        test["results"]["RMS_STABILITY"] = IVariation
        test["results"]["SHUNT_VOLTAGE"] = np.zeros(V.shape)
        test["defects"] = defects
        test["comments"] = comments

        # write attachment.
        if is_module:
            items = [
                "Type",
                "Wafer",
                "Module_SN",
                "Module_Stage",
                "Date",
                "Time",
                "Institute",
                "TestType",
                "Vbias_SMU",
                "Rseries",
                "Test_DMM",
                "Rshunt",
                "Software type and version, fw version",
                "RunNumber",
                "Temperature",
                "Humidity",
                "Comments",
            ]
        else:
            items = [
                "Type",
                "Batch",
                "Wafer",
                "Component",
                "Date",
                "Time",
                "Institute",
                "TestType",
                "Vbias_SMU",
                "Rseries",
                "Test_DMM",
                "Rshunt",
                "RunNumber",
                "Temperature",
                "Humidity",
                "Comments",
            ]

        try:
            fnam = "{}_{}_IV_{}-".format(
                self.mod_SN["double"], mdata["Module_Stage"], mdata["RunNumber"]
            )
        except KeyError:
            fnam = "{}_W_IV_{}".format(self.mod_SN["double"], mdata["RunNumber"])

        data_out = tempfile.NamedTemporaryFile(
            "w", prefix=fnam, suffix=".dat", delete=False
        )
        data_out.write("{}\n".format(fnam))
        for key in items:
            if key == "Module_SN" or key == "Component":
                data_out.write("{}: {}\n".format(key, self.mod_SN["double"]))
            else:
                data_out.write("{}: {}\n".format(key, mdata[key]))

        for il, label in enumerate(mdata["curve"]["labels"]):
            if il:
                data_out.write("\t")
            data_out.write(label)
        data_out.write("\n")

        ncol = len(mdata["curve"]["labels"])
        ndata = len(self.difference)
        for i in range(ndata):
            v = -abs(V[i])
            iv = -abs(self.difference[i])
            if ncol > 2:
                data_out.write("{:.2f}\t{:.2f}\t{:.2f}\n".format(v, iv, 0.0))
            else:
                data_out.write("{:.2f}\t{:.2f}\n".format(v, iv))

        print(data_out.name)
        data_out.close()

        js_out = tempfile.NamedTemporaryFile(
            "w", prefix="payload-", suffix=".json", delete=False
        )
        js_out.write(json.dumps(test, indent=3, cls=dbGtkUtils.MyEncoder))
        js_out.close()

        attachment = ITkDButils.Attachment(data_out.name, "resultsFile", fnam)
        uploadW = uploadTest.UploadTest(self.session, js_out.name, attachment)
        uploadW.connect("destroy", remove_files, [data_out.name, js_out.name])

    def on_refresh(self, *args):
        """Refresh canvas."""
        if self.fig and self.canvas:
            self.fig.tight_layout()
            self.canvas.draw()

    def find_module(self, SN):
        """Find module (SN) on database

        Args:
        ----
            SN (str): Module Serial number
        """
        md = ITkDButils.get_DB_component(self.session, SN)
        if md is None:
            dbGtkUtils.complain(
                "Could not find {}".format(SN), str(ITkDButils.get_db_response())
            )

        return md

    def on_single_file_set(self, *args):
        """File chosen."""
        obj_type = ["sensor", "module"]
        fnam = self.single_file.get_filename()
        if fnam is None or not Path(fnam).exists():
            dbGtkUtils.complain("Could not find data file", fnam, parent=self)

        mdata = self.read_file(fnam)

        is_module = 1
        try:
            SN = mdata["Module_SN"]
        except KeyError:
            SN = mdata["Component"]
            is_module = 0

        self.write_message("Reading data for {} {}\n".format(obj_type[is_module], SN))
        md = self.find_module(SN)
        if md is None:
            self.write_message("...object does not exist.\n")
            self.single_file.unselect_all()
            return

        # All good
        self.mod_SN["single"] = SN
        self.mdata["single"] = mdata
        self.mod_type["single"] = md["type"]["code"]
        print(self.mod_type["single"])

        self.single_SN.set_text("{} - {}".format(SN, md["type"]["name"]))
        self.show_curve(
            131,
            mdata["curve"]["V"],
            mdata["curve"]["I"],
            self.mod_type["single"][0:4] if is_module else "Single",
            mdata["curve"]["labels"][0],
            mdata["curve"]["labels"][1],
        )

    def on_double_file_set(self, *args):
        "File chosen for the 'double module'"
        obj_type = ["sensor", "module"]
        fnam = self.double_file.get_filename()
        if fnam is None or not Path(fnam).exists():
            dbGtkUtils.complain("Could not find data file", fnam, parent=self)

        mdata = self.read_file(fnam)
        is_module = 1
        # Check SN in data file
        try:
            SN = mdata["Module_SN"]
        except KeyError:
            is_module = 0
            SN = mdata["Component"]

        halfM_SN = SN
        if "single" in self.mod_SN:
            if self.mod_SN["single"] == SN:
                dbGtkUtils.complain(
                    "Wrong {}} SN", "{} already used.".format(obj_type[is_module], SN)
                )
                self.double_file.unselect_all()
                return

        # Check that it exists in the DB
        if len(SN) != 14 or SN["0:4"] != "20US":
            self.write_message("Invalid SN: {}\n".format(SN))
            SN = dbGtkUtils.get_a_value(
                "Invalid SN", "Give Ring or corresponding Half Module SN"
            )

        self.write_message("Reading data for module {}\n".format(SN))
        md = self.find_module(SN)
        if md is None:
            self.write_message("...object does not exist.\n")
            self.double_file.unselect_all()
            return

        found_child = False
        if md["type"]["name"].find("Ring") >= 0:
            self.write_message("...This is a Ring module. Searching children in DB\n")
            for child in md["children"]:
                if child["component"]:
                    ctype = child["type"]["code"]
                    if ctype.find("MODULE") < 0:
                        continue

                    cSN = child["component"]["serialNumber"]
                    if cSN == self.mod_SN["single"]:
                        continue

                    halfM_SN = cSN
                    found_child = True
                    self.write_message("...found {}\n".format(halfM_SN))
                    break

            if not found_child:
                self.write_message("Requesting a Half Module SN\n")
                halfM_SN = dbGtkUtils.get_a_value(
                    "Give Half Module SN", "Serial Number"
                )

            md = ITkDButils.get_DB_component(self.session, halfM_SN)
            if md is None:
                dbGtkUtils.complain(
                    "Could not find {}".format(halfM_SN),
                    str(ITkDButils.get_db_response()),
                )
                self.double_file.unselect_all()
                return

            self.write_message("... {}\n".format(halfM_SN))

        if "single" in self.mod_type:
            if is_module and self.mod_type["single"] == md["type"]["code"]:
                dbGtkUtils.complain(
                    "Wrong module type.",
                    "Module type cannot be {}".format(self.mod_type["single"]),
                )

                self.double_file.unselect_all()
                return

        self.mod_SN["double"] = halfM_SN
        self.mod_type["double"] = md["type"]["code"]
        self.mdata["double"] = mdata

        self.double_SN.set_text("{} - {}".format(halfM_SN, md["type"]["name"]))
        self.show_curve(
            133,
            mdata["curve"]["V"],
            mdata["curve"]["I"],
            "Double",
            mdata["curve"]["labels"][0],
            None,
        )

        # Compute difference if single already available
        if "single" in self.mdata:
            self.on_difference()

    def on_difference(self, *args):
        """Compute difference."""
        if "single" not in self.mdata or "double" not in self.mdata:
            dbGtkUtils.complain(
                "Data needed", "Check if single oand doubel module data are available"
            )
            return

        is_module = "Module_SN" in self.mdata["double"]
        double_I = self.mdata["double"]["curve"]["I"]
        single_I = scale_iv(
            self.mdata["single"]["curve"]["I"],
            self.mdata["single"]["Temperature"] + 273.0,
            self.mdata["double"]["Temperature"] + 273.0,
        )

        try:
            nmin = double_I.size
            self.difference = double_I - single_I
        except ValueError:
            nmin = np.min([double_I.size, single_I.size])
            self.write_message(
                "Size of current arrays is not the same: {} {}\n".format(
                    double_I.size, single_I.size
                )
            )
            self.difference = double_I[:nmin] - single_I[:nmin]

        self.show_curve(
            132,
            self.mdata["double"]["curve"]["V"][:nmin],
            self.difference,
            self.mod_type["double"][0:4] if is_module else "Diff",
            self.mdata["double"]["curve"]["labels"][0],
            None,
        )

    def show_curve(self, subplot, X, Y, title=None, xlabel="X", ylabel="Y"):
        """Shows data"""
        ax = self.fig.add_subplot(subplot)
        if xlabel:
            ax.set_xlabel(xlabel)

        if ylabel:
            ax.set_ylabel(ylabel)

        if title:
            ax.set_title(title)

        ax.plot(X, Y)
        ax.grid()
        self.on_refresh()

    @staticmethod
    def read_file(fnam):
        """Read a data file. Return dictionary with all teh data."""
        labels = []
        metadata = {}
        with open(fnam, "r", encoding="utf-8") as ifile:
            first = True
            for line in ifile:
                if first:
                    first = False
                    ipos = line.rfind(".")
                    metadata["fname"] = line[:ipos]
                    continue

                if line.find("Voltage [V]") >= 0 or line.find("Voltage[V]") >= 0:
                    labels = line.split("\t")
                    break

                rc = line.find(":")
                if rc >= 0:
                    key = line[:rc].strip()
                    val = line[rc + 1 :].strip()
                    if key in ["Temperature", "Humidity"]:
                        metadata[key] = float(val)
                    else:
                        metadata[key] = val

            V = []
            I = []
            for line in ifile:
                data = [float(s) for s in line.split()]
                V.append(data[0])
                I.append(data[1])

            metadata["curve"] = {
                "V": np.abs(np.array(V)),
                "I": np.abs(np.array(I)),
                "labels": labels[0:2],
            }
            return metadata


if __name__ == "__main__":
    import sys

    # DB login
    dlg = ITkDBlogin.ITkDBlogin()
    client = dlg.get_client()
    if client is None:
        print("Could not connect to DB with provided credentials.")
        dlg.die()
        sys.exit()

    client.user_gui = dlg

    # Start the Application
    win = IVwindow(client)
    win.show_all()
    win.set_accept_focus(True)
    win.present()
    win.connect("destroy", Gtk.main_quit)

    try:
        Gtk.main()
    except KeyboardInterrupt:
        print("Arrggggg !!!")

    dlg.die()
    print("Bye !!")
    sys.exit()



"""GUI to login to the DB."""
import getpass
import json
import os
import sys
import tempfile
import threading
import time
from pathlib import Path

import gi
import itkdb
from requests.status_codes import codes

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, GLib, GObject

gtk_runs, gtk_args = Gtk.init_check()


class TimeoutCaller(threading.Timer):
    """A recursive timer."""

    def run(self):
        """Thread function."""
        while not self.finished.wait(self.interval):
            self.function(*self.args, **self.kwargs)


def run_login_window(dlg):
    """Open the login dialog window in a callback."""
    if not dlg.has_window:
        dlg.create_window()

    dlg.in_callback = True
    dlg.ac1.set_text("")
    dlg.ac2.set_text("")
    dlg.show_all()
    dlg.run()
    dlg.hide()
    dlg.in_callback = False

    access1 = dlg.ac1.get_text()
    access2 = dlg.ac2.get_text()

    if access1 is None or access2 is None:
        dlg.complain("Bad Access codes.", "Not valid or empty access codes.")
        dlg.code = -2
        return

    dlg.user._access_code1 = access1
    dlg.user._access_code2 = access2
    dlg.has_to_wait = False

    dlg.set_new_credentials()
    return False


class ITkDBlogin(Gtk.Dialog):
    """Dialog to connect to the DB."""

    __gsignals__ = {
        'new_login': (GObject.SignalFlags.RUN_FIRST, None, (str, ))
    }

    def __init__(self, *args, **kwargs):
        """Initialization."""
        global gtk_runs
        if gtk_runs:
            super().__init__(*args, **kwargs)

        self.user = itkdb.core.User()
        self.code = -1
        self.has_to_wait = False
        self.in_callback = False
        self.token = None
        self.id_token = None
        self.has_window = False
        self.token_file = Path(tempfile.gettempdir(), ".itk_dbtoken")
        self.T = TimeoutCaller(30, self.reconnect)

        if os.path.exists(self.token_file):
            with open(self.token_file, 'r') as token_file:
                self.token = json.loads(token_file.read())

            if self.token:
                self.setToken()

        if self.token:
            self.T.start()
            return

        if gtk_runs:
            self.create_window()
            self.present()
            self.run()

        else:
            access1 = getpass.getpass("Access 1: ")
            access2 = getpass.getpass("Access 2: ")
            self.user._access_code1 = access1
            self.user._access_code2 = access2
            try:
                self.user.authenticate()
            except Exception:
                print("Could not connect to DB with provided credentials.")
                sys.exit()

            self.token = {
                "access_token": self.user._access_token,
                "id_token": self.user._raw_id_token,
                "issued_at": float(time.time()),
                "expires_at": self.user.expires_at
            }
            with open(self.token_file, 'w') as token_file:
                token_file.write(json.dumps(self.token))

    def do_new_login(self, *args):
        """New login."""
        pass

    def die(self):
        """Kill the timer."""
        self.T.cancel()

    def is_connected(self):
        """Tell if there is connection to DB."""
        return self.token is not None

    def reconnect(self, check=True, force=False):
        """REconnect."""
        if not force and check:
            if not self.checkTokenExpired():
                return

        set_new_cred = True
        if force or (self.user._access_code1 == "" or self.user._access_code2 == ""):

            if Gtk.main_level():
                self.has_to_wait = True
                GLib.idle_add(run_login_window, self)
                set_new_cred = False
                while not force and self.has_to_wait:
                    time.sleep(0.1)

            else:
                access1 = getpass.getpass("Access 1: ")
                access2 = getpass.getpass("Access 2: ")
                self.user._access_code1 = access1
                self.user._access_code2 = access2

            if self.code == -2:
                self.code = -1
                print("User cancelled login")
                self.die()
                sys.exit()

        if set_new_cred:
            self.set_new_credentials()

    def set_new_credentials(self):
        """Set new credentials."""
        self.user._access_token = None
        self.user._raw_id_token = None
        self.user._id_token = None
        try:
            self.user.authenticate()
            self.token = {
                "access_token": self.user._access_token,
                "id_token": self.user._raw_id_token,
                "issued_at": float(time.time()),
                "expires_at": self.user.expires_at
            }

            with open(self.token_file, 'w') as token_file:
                token_file.write(json.dumps(self.token))

            self.emit("new_login", "<OK>")

        except Exception as E:
            self.emit("new_login", str(E))

    def setToken(self):
        """Set the token things."""
        if self.checkTokenExpired():
            self.token = None
            return

        self.user._status_code = codes['ok']
        self.user._access_token = self.token['access_token']
        self.user._raw_id_token = self.token['id_token']
        self.user._id_token = self.user._raw_id_token
        self.user._parse_id_token()
        self.user._session.headers.update({'Authorization': 'Bearer {0:s}'.format(self.user.bearer)})

    def checkTokenExpired(self):
        """Check if token is expired."""
        if self.token is None:
            return True

        try:
            if not self.user.is_authenticated():
                delta = self.token['expires_at'] - time.time() - 2*self.T.interval
            else:
                delta = self.user.expires_in - 2 * self.T.interval

            return delta <= 0

        except KeyError:
            return True

    def create_window(self):
        """Create the window."""
        # set border width
        self.set_border_width(10)

        #
        self.has_window = True
        self.add_buttons(
            Gtk.STOCK_OK, Gtk.ResponseType.OK, Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL
        )
        # self.set_position(Gtk.WindowPosition.CENTER_ALWAYS)

        self.set_title("Login to the ITk DB")

        mainBox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        self.get_content_area().add(mainBox)

        title = Gtk.Label()
        title.set_markup('<b>ATLAS ITk Production Database Login</b>')
        mainBox.pack_start(title, True, True, 5)

        grid = Gtk.Grid(column_spacing=5, row_spacing=5)
        mainBox.pack_start(grid, True, True, 5)

        irow = 0
        self.ac1 = Gtk.Entry()
        self.ac2 = Gtk.Entry()
        self.ac1.set_visibility(False)
        self.ac2.set_visibility(False)

        grid.attach(Gtk.Label(label="Acces Code 1"), 0, irow, 1, 1)
        grid.attach(self.ac1, 1, irow, 1, 1)
        irow = irow + 1
        grid.attach(Gtk.Label(label="Acces Code 1"), 0, irow, 1, 1)
        grid.attach(self.ac2, 1, irow, 1, 1)

        btn = self.get_widget_for_response(Gtk.ResponseType.OK)
        btn.connect("clicked", self.on_run)

        self.show_all()

    def complain(self, main_title, second_text):
        """Opens an error dialog.

        Args:
        ----
            main_title: Main text in window
            second_text: Second text

        """
        dialog = Gtk.MessageDialog(
            transient_for=self,
            flags=0,
            message_type=Gtk.MessageType.ERROR,
            buttons=Gtk.ButtonsType.OK,
            text=main_title,
        )
        dialog.format_secondary_text(second_text)
        dialog.run()
        dialog.destroy()

    @property
    def name(self):
        """Return User DB name."""
        return self.user.name

    def on_run(self, widget):
        """OK button clicked."""
        if self.in_callback:
            return

        print("logging in")
        access1 = self.ac1.get_text()
        access2 = self.ac2.get_text()

        if access1 is None or access2 is None:
            self.complain("Bad Access codes.", "Not valid or empty access codes.")
            self.code = -2
            return

        else:
            self.user._access_code1 = access1
            self.user._access_code2 = access2
            self.reconnect(False)
            if not self.T.is_alive():
                self.T.start()
            # self.user.authenticate()
            # self.token = {
            #     "access_token": self.user._access_token,
            #     "id_token": self.user._raw_id_token,
            #     "issued_at": float(time.time()),
            #     "expires_at": self.user.expires_at
            # }

            # with open('.itk_dbtoken', 'w') as token_file:
            #     token_file.write(json.dumps(self.token))
            self.hide()

    def get_client(self):
        """Return the client."""
        if not self.is_connected():
            return None

        return itkdb.Client(user=self.user)

    def __del__(self):
        """Delete."""
        self.die()


if __name__ == "__main__":
    # Initialize the session in the ITk PDB
    dlg = ITkDBlogin()
    client = dlg.get_client()
    if client is None:
        print("Could not connect to DB with provided credentials.")
        sys.exit()

    # payload = {
    #     "project": "S",
    #     "componentType": "MODULE",
    #     "testType": "GLUE_WEIGHT"
    # }
    # lst = client.get("listTestRunsByTestType", json=payload)
    # for l in enumerate(lst):
    #     print(l)

    print("Hello {}".format(dlg.name))
    if gtk_runs:
        try:
            while True:
                time.sleep(0.25)

        except Exception:
            print("Arrgs")

    else:
        print("Logged on an ascii terminal.")

    dlg.die()
